searchNodes=[{"ref":"ClosedIntervals.html","title":"ClosedIntervals","type":"module","doc":"A ClosedIntervals datastructure. ClosedIntervals represents a set of closed intervals and provides functions to retrieve the interval to which a given value belongs to. ClosedIntervals can handle arbitrary data, as long as it can be ordered in a sensible way. Users can either use the default term order &amp;&lt;=/2 if that suits their needs, or provide an explicit order function."},{"ref":"ClosedIntervals.html#closed_intervals/1","title":"ClosedIntervals.closed_intervals/1","type":"macro","doc":"This is the internal tree representation. It is not intended to be used publicly."},{"ref":"ClosedIntervals.html#closed_intervals/2","title":"ClosedIntervals.closed_intervals/2","type":"macro","doc":""},{"ref":"ClosedIntervals.html#from/2","title":"ClosedIntervals.from/2","type":"function","doc":"Create a new ClosedIntervals from points. This function creates a new ClosedIntervals from an Enum of points. The points can be of any form, as long as they can be ordered sensibly. For types where the term order does not order them in a way such that the resulting order represents a linear ordering along the interval range, a custom order can be applied using the order parameter. order defaults to &amp;&lt;=/2. Note that a custom order should return true for equal points, if the resulting order has to be stable. Additionally, an explicit equality function can be provided which is used in ClosedIntervals.get_interval/2 and ClosedIntervals.get_all_intervals/2. Errors The function expects that the enum contains at least two points. If that is not the case, an ArgumentError is raised. iex&gt; from([1]) ** (ArgumentError) Need at least two points to construct an ClosedIntervals Examples from/1,2 can handle plain types: iex&gt; from([1, 2, 3]) |&gt; leaf_intervals() [{1, 2}, {2, 3}] It can also handle nested types, if a suitable order is defined: iex&gt; points = [%{idx: 3}, %{idx: 7}, %{idx: 1}] iex&gt; points |&gt; from(order: &amp;(&amp;1.idx &lt;= &amp;2.idx)) |&gt; leaf_intervals() [{%{idx: 1}, %{idx: 3}}, {%{idx: 3}, %{idx: 7}}] Arguments :order: A custom order defined on the points used to construct the ClosedIntervals :eq: A custom equality defined on the points used to construct the ClosedIntervals"},{"ref":"ClosedIntervals.html#get_all_intervals/2","title":"ClosedIntervals.get_all_intervals/2","type":"function","doc":"Retrieve all intervals which cover value. This function is useful if the index points used to define the ClosedIntervals are not unique. For example, when defining a step-function, it might make sense to use the same point multiple times but with different data in order to represent a sharp step. Values which are placed right at the interval bounds can then belong to multiple closed intervals."},{"ref":"ClosedIntervals.html#get_interval/2","title":"ClosedIntervals.get_interval/2","type":"function","doc":"Get the interval to which a value belongs to. Example iex&gt; closed_intervals = from([1, 2, 5]) iex&gt; get_interval(closed_intervals, 3) {2, 5}"},{"ref":"ClosedIntervals.html#leaf_intervals/1","title":"ClosedIntervals.leaf_intervals/1","type":"function","doc":"Retrieve a list of all leaf intervals. A leaf interval is an interval which has been constructed from two adjacent points. It does not expand to :&quot;-inf&quot; or :&quot;+inf&quot;. Example iex&gt; from([1, 2, 3]) |&gt; leaf_intervals() [{1, 2}, {2, 3}]"},{"ref":"ClosedIntervals.html#left_bound/1","title":"ClosedIntervals.left_bound/1","type":"function","doc":"Retrieve the left bound of a ClosedIntervals. Example iex&gt; [1, 2, 3] |&gt; from() |&gt; left_bound() 1"},{"ref":"ClosedIntervals.html#map/2","title":"ClosedIntervals.map/2","type":"function","doc":"Map a function over all intervals. Example iex&gt; closed_intervals = from([1, 2, 3]) iex&gt; map(closed_intervals, &amp; &amp;1 + 1) |&gt; to_list() [2, 3, 4]"},{"ref":"ClosedIntervals.html#right_bound/1","title":"ClosedIntervals.right_bound/1","type":"function","doc":"Retrieve the right bound of a ClosedIntervals. Example iex&gt; [1, 2, 3] |&gt; from() |&gt; right_bound() 3"},{"ref":"ClosedIntervals.html#to_list/1","title":"ClosedIntervals.to_list/1","type":"function","doc":"Serialize ClosedIntervals into a list. Example iex&gt; closed_intervals = from([1, 2, 3]) iex&gt; to_list(closed_intervals) [1, 2, 3] iex&gt; closed_intervals == closed_intervals |&gt; to_list() |&gt; from() true"},{"ref":"ClosedIntervals.html#t:t/1","title":"ClosedIntervals.t/1","type":"type","doc":""},{"ref":"ClosedIntervals.html#t:tree/1","title":"ClosedIntervals.tree/1","type":"type","doc":""},{"ref":"readme.html","title":"ClosedIntervals","type":"extras","doc":"ClosedIntervals ClosedIntervals implements a tree storing a set of closed intervals such that the intervals cover a one-dimensional space completely. So for a set of points {x1, x2, .., xN}, ClosedIntervals builds a new set {(:&quot;-inf&quot;, x1), (x1, x2), ..., (xN-1, xN), (xN, :&quot;+inf&quot;)} and provides functions to retrieve the interval to which a value belongs."},{"ref":"readme.html#installation","title":"ClosedIntervals - Installation","type":"extras","doc":"Add the closed_intervals dependency to your mix.exs: def deps do [ {:closed_intervals, &quot;~&gt; 0.3&quot;} ] end"},{"ref":"readme.html#examples","title":"ClosedIntervals - Examples","type":"extras","doc":"As a simple example, consider the following set of points: [1, 10, 30] With ClosedIntervals, the points above are placed on a number line. Now, for any value on that number line, we can retrieve the interval to which that value belongs: iex&gt; import ClosedIntervals iex&gt; closed_intervals = from([1, 10, 30]) iex&gt; get_interval(closed_intervals, 2) {1, 10} iex&gt; get_interval(closed_intervals, 11) {10, 30} iex&gt; get_interval(closed_intervals, -1) {:&quot;-inf&quot;, 1} iex&gt; get_interval(closed_intervals, 1) {:&quot;-inf&quot;, 1} iex&gt; get_interval(closed_intervals, 100) {30, :&quot;+inf&quot;} So if we retrieve the interval for 2, we receive {1, 10}, as the value 2 is between 1 and 10. If we query -1, we receive {:&quot;-inf&quot;, 1}, as -1 is in front of all defined intervals. Similarly for 100, we retrieve {30, :&quot;+inf&quot;}. Defining a linear space with plain numbers is boring, however. Where ClosedIntervals shines is to define an order on multi-dimensional values, where an order in one dimension makes sense. This is done by defining an explicit order on ClosedIntervals.from/2 and ClosedIntervals.get_interval/2: iex&gt; import ClosedIntervals iex&gt; points = [%{idx: 1, data: :hello}, %{idx: 5, data: :world}] iex&gt; order = fn a, b -&gt; a.idx &lt;= b.idx end iex&gt; closed_intervals = from(points, order: order) iex&gt; get_interval(closed_intervals, %{idx: 3}) {%{idx: 1, data: :hello}, %{idx: 5, data: :world}} ClosedIntervals can also handle non-unique indices. This is useful when defining a function step-wise. Note that in such a case, the intervals for a value should be retrieved using ClosedIntervals.get_all_intervals/2, as a value may belong to more than one interval. For this, we must define an equality function as well. Usually, this function compares the same fields as the order function. iex&gt; import ClosedIntervals iex&gt; points = [%{idx: 1, data: :hello}, %{idx: 1, data: :between}, %{idx: 5, data: :world}] iex&gt; order = fn a, b -&gt; a.idx &lt;= b.idx end iex&gt; eq = fn a, b -&gt; a.idx == b.idx end iex&gt; closed_intervals = from(points, order: order, eq: eq) iex&gt; get_all_intervals(closed_intervals, %{idx: 3}) [{%{idx: 1, data: :between}, %{idx: 5, data: :world}}] iex&gt; get_all_intervals(closed_intervals, %{idx: 1}) [ {:&quot;-inf&quot;, %{idx: 1, data: :hello}}, {%{idx: 1, data: :hello}, %{idx: 1, data: :between}}, {%{idx: 1, data: :between}, %{idx: 5, data: :world}} ]"},{"ref":"readme.html#internals","title":"ClosedIntervals - Internals","type":"extras","doc":"ClosedIntervals uses a tree of tuples to represent the set of intervals. Using tuples is considerably faster than using maps, and it is faster than running a simple linear search on a list of intervals. A linear search would outperform the implementation here as long as the value queried for is at the beginning of the list, but the performance degrades very fast for long lists. See the benchmark/ for a simple benchmark of a linear search compared to ClosedIntervals."},{"ref":"readme.html#license","title":"ClosedIntervals - LICENSE","type":"extras","doc":"ClosedIntervals is released under Apache License 2.0. See the LICENSE file for more information."},{"ref":"changelog.html","title":"Changelog","type":"extras","doc":"Changelog"},{"ref":"changelog.html#0-3-0","title":"Changelog - 0.3.0","type":"extras","doc":"Fix get_all_intervals/2 Add left_bound/1 and right_bound/1"},{"ref":"changelog.html#0-2-0","title":"Changelog - 0.2.0","type":"extras","doc":"Add Typespecs Add ClosedIntervals.map/2"},{"ref":"changelog.html#0-1-1","title":"Changelog - 0.1.1","type":"extras","doc":"Initial release"},{"ref":"changelog.html#0-1-0","title":"Changelog - 0.1.0","type":"extras","doc":"Initial version Implements ClosedIntervals ClosedIntervals.from/1,2 ClosedIntervals.get_interval/2 ClosedIntervals.get_all_intervals/2"}]